//
//  ChimpanzeeGameAppDelegate.m
//  ChimpanzeeGame
//
//  Created by kunii on 08/10/13.
//  Copyright __MyCompanyName__ 2008. All rights reserved.
//

#import "ChimpanzeeGameViewController.h"

static const int SEQUENCE_TERMINATOR = -1;	//	clickSquence[index]の値が、この値の場合終了。

@implementation ChimpanzeeGameViewController

/*
	ボタンクリック対応
*/
- (IBAction)respondToButtonClick:(UIButton*)sender 
{
    NSInteger tag = sender.tag;		//	タグ取りだし
	if (clickSquence[index] == tag) {
		sender.hidden = TRUE;		//	正解のボタンを押された場合、そのボタンを隠す。
		index++;					//	次の順序に進める。
	} else {
		[self start:@"Faled!"];		//	間違いというアラートを表示してゲーム再開
		return;
	}
	if (clickSquence[index] == SEQUENCE_TERMINATOR) {
		[self start:@"Complite!"];	//	成功というアラートを表示してゲーム再開
	}
}


/*
 Implement loadView if you want to create a view hierarchy programmatically
- (void)loadView {
}
 */

/*
 Implement viewDidLoad if you need to do additional setup after loading the view.
*/
- (void)viewDidLoad 
{
	static const int BUTTON_WIDTH = 80;		//	ボタンの幅
	static const int BUTTON_HEIGHT = 80;	//	ボタンの高さ
	static const int BUTTON_MERGIN = 10;	//	ボタン間隔
	[super viewDidLoad];
	int i = 0;
	for (int v = 0; v < BUTTON_V_COUNT; v++) { 
		for (int h = 0; h < BUTTON_H_COUNT; h++) { 
			UIButton* bt = [[UIButton buttonWithType:UIButtonTypeRoundedRect] retain];
			bt.frame = CGRectMake(			//	ボタン位置設定
				BUTTON_MERGIN + h * (BUTTON_WIDTH + BUTTON_MERGIN), 
				BUTTON_MERGIN + v * (BUTTON_HEIGHT + BUTTON_MERGIN), 
				BUTTON_WIDTH, BUTTON_HEIGHT);
			[bt addTarget:self action:@selector(respondToButtonClick:)	//	アクション設定
				forControlEvents:UIControlEventTouchUpInside];
			bt.tag = i;						//	タグ設定
			bt.hidden = TRUE;				//	最初は非表示
			bt.enabled = FALSE;				//	最初はボタンを押せない状態にする。
			[self.view addSubview:bt];			//	ビューに登録
			buttons[i++] = bt;				//	自身の変数に登録
		}
	}
	[self start:@""];						//	開始
 }

/*
	全ボタンを隠す
*/
- (void)hideAllButtons
{
	int i = 0;
	for (int v = 0; v < BUTTON_V_COUNT; v++) { 
		for (int h = 0; h < BUTTON_H_COUNT; h++) { 
			buttons[i++].hidden = TRUE;
		}
	}
}

/*
	messageをアラートで表示してゲーム開始。
*/
- (void)start:(NSString*)message
{
	[self hideAllButtons];					//	全ボタンを隠す
	//	いったん手動で順序設定
	clickSquence[0] = 2;
	clickSquence[1] = 8;
	clickSquence[2] = 6;
	clickSquence[3] = SEQUENCE_TERMINATOR;
	index = 0;								//	チェックする順序を先頭にする。
	//	開始画面表示
	UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"chimpanzee!" 
		message:message
		delegate:self cancelButtonTitle:@"Start" otherButtonTitles: nil];
	[alert show];	
	[alert release];	
}

/*
	アラートクリック後の処理
	
	クリックする順番を、1〜（人の感覚に合うよう、0ではなく1から始める）としてボタン自身に表示させる。
*/
- (void)alertView:(UIAlertView *)actionSheet clickedButtonAtIndex:(NSInteger)buttonIndex
{
	static const double SHOW_SEQUENCE_TIME = 2.0;	//	クリック順を見せている時間。
	for (int i = 0 ; clickSquence[i] != SEQUENCE_TERMINATOR; i++) {
		UIButton* bt = buttons[clickSquence[i]];
		[bt setTitle:[NSString stringWithFormat:@"%d", i + 1] forState:UIControlStateNormal];
		bt.hidden = FALSE;		//	表示
		bt.enabled = FALSE;		//	ただし、押されて反応させないようにする。
		CGAffineTransform scaling = CGAffineTransformMakeScale(1.0, 1.0);
		bt.transform = scaling;
		bt.alpha = 1.0;
		CGAffineTransform scaling2 = CGAffineTransformMakeScale(2.5, 2.5);
		[UIView beginAnimations:nil context:NULL];
		[UIView setAnimationDuration:1.0];
		[UIView setAnimationDelay:i * 0.5];
		bt.transform = scaling2;
		bt.alpha = 0.0;
		[UIView commitAnimations];
	}
	//	タイマーでSHOW_SEQUENCE_TIME秒後にボタンを消えるようにする
	[[NSTimer scheduledTimerWithTimeInterval:SHOW_SEQUENCE_TIME
		target:self selector:@selector(hideClickSequence:) userInfo:0 repeats:NO] retain];
}

/*
	いったん全ボタンを消し、0.5秒後にボタンの順序を隠した状態で表示。
*/
- (void)hideClickSequence:( NSTimer *)aTimer
{
	static const double BLACKOUT_TIME = 0.5;	//	全ボタンが消えている時間。
	[self hideAllButtons];						//	全ボタンを隠す。見えているのは一部だが...
	[aTimer release];							//	タイマー開放
	//	新しいタイマーでBLACKOUT_TIME秒後にボタンを消えるようにする
	[[NSTimer scheduledTimerWithTimeInterval:BLACKOUT_TIME
		target:self selector:@selector(startGame:) userInfo:0 repeats:NO] retain];
}

/*
	ゲーム開始
*/
- (void)startGame:( NSTimer *)aTimer
{
	for (int i = 0 ; clickSquence[i] != SEQUENCE_TERMINATOR; i++) {
		UIButton* bt = buttons[clickSquence[i]];
		[bt setTitle:@"" forState:UIControlStateNormal];	//	タイトルは空にする。
		[UIView commitAnimations];
		CGAffineTransform scaling = CGAffineTransformMakeScale(1.0, 1.0);
		bt.transform = scaling;
		bt.alpha = 1.0;
		bt.hidden = FALSE;						//	表示
		bt.enabled = TRUE;						//	今回はクリックを受け付ける。
	}
	[aTimer release];							//	タイマー開放
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
	// Return YES for supported orientations
	return (interfaceOrientation == UIInterfaceOrientationPortrait);
}


- (void)didReceiveMemoryWarning {
	[super didReceiveMemoryWarning]; // Releases the view if it doesn't have a superview
	// Release anything that's not essential, such as cached data
}

/*
	終了
	利用していたボタンを開放する。
*/
- (void)dealloc {
	int i = 0;	//	ボタンインディックス
	for (int v = 0; v < BUTTON_V_COUNT; v++) { 
		for (int h = 0; h < BUTTON_H_COUNT; h++) {
			[buttons[i++] release];	//	開放
		}
	}
	[super dealloc];
}

@end
